//===================================================================
// asb3000s.c
//-------------------------------------------------------------------
//
// ASB-3000 Sample
//
//-------------------------------------------------------------------
// CH1CALM͂AgKʒȗÕf[^\܂B
//===================================================================

#include <windows.h>
#include <tchar.h>
#include "asb3000.h"
#include "asb3000w.h"


#define WM_ASB3000_BASE			WM_USER + 100
#define WM_ASB3000_DATA_READY	WM_ASB3000_BASE + WM_DATA_READY
#define WM_ASB3000_ERROR		WM_ASB3000_BASE + WM_DATA_THREAD_ERROR

const TCHAR szTitle[] = _T( "Sample" );
const TCHAR szClassName[] = _T( "SampleProgram" );

static void ResultMessage( LPCTSTR lpszbuf );
static void ErrorMessage( WORD devno );
double VoltValue( DWORD addata );

LRESULT CALLBACK WndProc( HWND, UINT, WPARAM, LPARAM );

BOOL	OnCreate( void );
BOOL	OnDestroy( void );
BOOL	DataReady( WPARAM, LPARAM );
BOOL	DataThreadError( WPARAM, LPARAM );

HWND		mhWnd;
WORD		mwDeviceNo;
PDWORD		mpdwADData;
TSETMODE	mSetMode;

//-------------------------------------------------------------------
// Main
//-------------------------------------------------------------------
int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
					LPSTR lpCmdLine, int nCmdShow )
{
	MSG			msg;
	WNDCLASS	wc;

	//---------------------------------------------------------------
	// EChENXo^
	//---------------------------------------------------------------
	if ( !hPrevInstance ) {
		wc.style		 = CS_HREDRAW | CS_VREDRAW;
		wc.lpfnWndProc	 = WndProc;
		wc.cbClsExtra	 = 0;
		wc.cbWndExtra	 = 0;
		wc.hInstance	 = hInstance;
		wc.hIcon		 = NULL;
		wc.hCursor		 = LoadCursor( NULL, IDC_ARROW );
		wc.hbrBackground = GetStockObject( WHITE_BRUSH );
		wc.lpszMenuName	 = NULL;
		wc.lpszClassName = szClassName;
		if ( !RegisterClass( &wc ) )
			return FALSE;
	}
	//---------------------------------------------------------------
	// EChE쐬
	//---------------------------------------------------------------
	mhWnd = CreateWindow( szClassName,
						  szTitle,
						  WS_OVERLAPPEDWINDOW,
						  CW_USEDEFAULT,
						  CW_USEDEFAULT,
						  150,
						  150,
						  NULL,
						  NULL,
						  hInstance,
						  NULL );
	if ( mhWnd == NULL )
		return FALSE;
	//---------------------------------------------------------------
	// EChE\
	//---------------------------------------------------------------
	ShowWindow( mhWnd, nCmdShow );
	UpdateWindow( mhWnd );
	//---------------------------------------------------------------
	// bZ[W[v
	//---------------------------------------------------------------
	while ( GetMessage( &msg, NULL, 0, 0 ) ) {
		TranslateMessage( &msg );
		DispatchMessage( &msg );
	}
	return ( (int)msg.wParam );
}
//-------------------------------------------------------------------
// EChEbZ[W
//-------------------------------------------------------------------
LRESULT CALLBACK WndProc( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch ( msg ) {
		case WM_SHOWWINDOW:
			OnCreate();
			break;
		case WM_ASB3000_DATA_READY:
			DataReady( wParam, lParam );
			break;
		case WM_ASB3000_ERROR:
			DataThreadError( wParam, lParam );
			break;
		case WM_DESTROY:
			OnDestroy();
			PostQuitMessage( 0 );
			break;
		default:
			return ( DefWindowProc( hwnd, msg, wParam, lParam ) );
	}
	return 0;
}
//-------------------------------------------------------------------
// 
//-------------------------------------------------------------------
BOOL OnCreate( void )
{
	TCHAR		szbuf[ 1024 ];
	DWORD		dwDrvVer;
	DWORD		dwDllVer;
	DWORD		dwMemSize;
	TDEVINFO	DevInfo;

	//----------------------------------------------------------
	// DLL̎gp錾
	//----------------------------------------------------------
	if ( FALSE == Asb3000wDllOpen() ) {
		ErrorMessage( ASB3000_DEVICE_AUTO );
	}
	//----------------------------------------------------------
	// foCX̎gp錾
	//----------------------------------------------------------
	mwDeviceNo = ASB3000_DEVICE_AUTO;
	if ( FALSE == Asb3000wCreate( &mwDeviceNo, mhWnd, WM_ASB3000_BASE ) ) {
		ErrorMessage( ASB3000_DEVICE_AUTO );
	}
	_stprintf_s( szbuf, 1024, _T( "foCX %d ̎gp錾܂B" ), mwDeviceNo );
	ResultMessage( szbuf );
	//----------------------------------------------------------
	// o[W擾
	//----------------------------------------------------------
	if ( FALSE == Asb3000wGetVersion( &dwDrvVer, &dwDllVer ) ) {
		ErrorMessage( ASB3000_DEVICE_AUTO );
	}
	_stprintf_s( szbuf, 1024, _T( "o[W\n")
							  _T( "   Driver Version : %.8lxH\n" )
							  _T( "   DLL Version : %.8lxH" ),
							  dwDrvVer, dwDllVer );
	ResultMessage( szbuf );
	//------------------------------------------------------------
	// foCX擾
	//------------------------------------------------------------
	if ( FALSE == Asb3000wGetDevInfo( mwDeviceNo, &DevInfo ) ) {
		ErrorMessage( mwDeviceNo );
	}
	_stprintf_s( szbuf, 1024, _T( "foCX\n" )
							  _T( "   Firm Version : %.2f\n" )
							  _T( "   FPGA Version : %.2f" ),
							  DevInfo.FirmVer / 100.0,
							  DevInfo.FpgaVer / 100.0);
	ResultMessage( szbuf );
	//------------------------------------------------------------
	// TCY擾
	//------------------------------------------------------------
	if ( FALSE == Asb3000wGetMemSize( mwDeviceNo, &dwMemSize ) ) {
		ErrorMessage( mwDeviceNo );
	}
	_stprintf_s( szbuf, 1024, _T( "TCY\n" ) );
	if ( 0 == dwMemSize ) {
		_tcscat_s( szbuf, _countof(szbuf), _T( "   128K|Cg" ) );
	}
	else {
		_tcscat_s( szbuf, _countof(szbuf), _T( "   1M|Cg" ) );
	}
	ResultMessage( szbuf );
	//------------------------------------------------------------
	// TvO샂[hݒ
	//------------------------------------------------------------
	// 
	mSetMode.dwVoltDiv[ ASB3000_CH1 ]	   = ASB3000_VOLTDIV_1;		// XP[
	mSetMode.dwVoltDiv[ ASB3000_CH2 ]	   = ASB3000_VOLTDIV_1;
	mSetMode.dwVoltPos[ ASB3000_CH1 ]	   = 100;					// |WV
	mSetMode.dwVoltPos[ ASB3000_CH2 ]	   = 100;
	mSetMode.dwVertCoupling[ ASB3000_CH1 ] = ASB3000_CUP_DC;		// ͌
	mSetMode.dwVertCoupling[ ASB3000_CH2 ] = ASB3000_CUP_DC;
	// 
	mSetMode.dwMemSize					   = ASB3000_MEMSIZE_10K;	// TCY
	mSetMode.dwSamplingTime				   = ASB3000_TIMEDIV_200NS;	// XP[
	mSetMode.dwBandWidth[ ASB3000_CH1 ]    = ASB3000_BW_FULL;		// gш
	mSetMode.dwBandWidth[ ASB3000_CH2 ]    = ASB3000_BW_FULL;
	mSetMode.dwSamplingMode				   = ASB3000_SMP_REAL;		// TvO[h
	mSetMode.dwAcqMode					   = ASB3000_ACQ_SINGLE;	// ANCWV[h
	// gK
	mSetMode.dwTrigMode					   = ASB3000_TMODE_AUTO;	// gK[h
	mSetMode.dwTrigSource				   = ASB3000_TSRC_CH1;		// gK\[X
	mSetMode.dwTrigCoupling				   = ASB3000_TCUP_DC;		// gK͌
	mSetMode.dwTrigSlope				   = ASB3000_TSLP_POS;		// gKX[v
	mSetMode.dwTrigLevel				   = 250;					// gKx
	mSetMode.dwTrigPos					   = 5000;					// gK|WV
	mSetMode.dwTrigDelay				   = 0;						// gKfBC
	mSetMode.dwTrigPulse				   = ASB3000_PULSE_NONE;	// pXgK(pX)
	mSetMode.dwTrigPulseWidth			   = 0;						// pXgK(pX)
	if ( FALSE == Asb3000wSetMode( mwDeviceNo, &mSetMode ) ) {
		ErrorMessage( mwDeviceNo );
	}
	_stprintf_s( szbuf, 1024, _T( "TvO샂[hݒ\n" )
							  _T( " yz\n" )
							  _T( "   CH1 XP[ : %d\n" )
							  _T( "   CH2 XP[ : %d\n" )
							  _T( "   CH1 |WV : %d\n" )
							  _T( "   CH2 |WV : %d\n" )
							  _T( "   CH1 ͌ : %d\n" )
							  _T( "   CH2 ͌ : %d\n" )
							  _T( " yz\n" )
							  _T( "   TCY : %d\n" )
							  _T( "   TvO^C : %d\n" )
							  _T( "   CH1 gш : %d\n" )
							  _T( "   CH2 gш : %d\n" )
							  _T( "   TvO[h : %d\n" )
							  _T( "   ANCWV[h : %d\n" )
							  _T( " ygKz\n" )
							  _T( "   [h : %d\n" )
							  _T( "   \[X : %d\n" )
							  _T( "   ͌ : %d\n" )
							  _T( "   X[v : %d\n" )
							  _T( "   x : %d\n" )
							  _T( "   |WV : %d\n" )
							  _T( "   fBC : %d\n" )
							  _T( "   pX : %d\n" )
							  _T( "   pX : %d" ),
							  mSetMode.dwVoltDiv[ ASB3000_CH1 ],
							  mSetMode.dwVoltDiv[ ASB3000_CH2 ],
							  mSetMode.dwVoltPos[ ASB3000_CH1 ],
							  mSetMode.dwVoltPos[ ASB3000_CH2 ],
							  mSetMode.dwVertCoupling[ ASB3000_CH1 ],
							  mSetMode.dwVertCoupling[ ASB3000_CH2 ],
							  mSetMode.dwMemSize,
							  mSetMode.dwSamplingTime,
							  mSetMode.dwBandWidth[ ASB3000_CH1 ],
							  mSetMode.dwBandWidth[ ASB3000_CH2 ],
							  mSetMode.dwSamplingMode,
							  mSetMode.dwAcqMode,
							  mSetMode.dwTrigMode,
							  mSetMode.dwTrigSource,
							  mSetMode.dwTrigCoupling,
							  mSetMode.dwTrigSlope,
							  mSetMode.dwTrigLevel,
							  mSetMode.dwTrigPos,
							  mSetMode.dwTrigDelay,
							  mSetMode.dwTrigPulse,
							  mSetMode.dwTrigPulseWidth );
	ResultMessage( szbuf );
	//------------------------------------------------------------
	// TvOJn
	//------------------------------------------------------------
	ResultMessage( _T("TvOJn܂B") );
	if ( FALSE == Asb3000wStartSampling( mwDeviceNo, 0 ) ) {
		ErrorMessage( mwDeviceNo );
	}
	return TRUE;
}
//-------------------------------------------------------------------
// I
//-------------------------------------------------------------------
BOOL OnDestroy( void )
{
	//---------------------------------------------------------------
	// foCX̎gpJ
	//---------------------------------------------------------------
	if ( FALSE == Asb3000wClose( mwDeviceNo ) ) {
		ErrorMessage( mwDeviceNo );
	}
	ResultMessage( _T("foCXJ܂B") );
	//---------------------------------------------------------------
	// DLL̎gpJ
	//---------------------------------------------------------------
	if ( FALSE == Asb3000wDllClose() ) {
		MessageBox( NULL, _T( "asb3000.dll ̃A[hɎs܂B" ),
					szTitle, MB_OK | MB_ICONSTOP );
		return FALSE;
	}		
	return TRUE;
}
//-------------------------------------------------------------------
// `f[^
//-------------------------------------------------------------------
BOOL DataReady( WPARAM wParam, LPARAM lParam )
{

	TCHAR	szbuf[ 1024 ];
	TCHAR	sztmp[ 1024 ];
	WORD	wDevNo;
	DWORD	dwStatus;
	DWORD	dwNumData;
	DWORD	dwInitArg;
	DWORD	dwData;
	int		i;

	if ( ASB3000_DEVICE_ID != lParam ) {
		return FALSE;
	}
	wDevNo = (WORD)wParam;
	//----------------------------------------------------------
	// TvO~
	//----------------------------------------------------------
	if ( FALSE == Asb3000wStopSampling( wDevNo ) ) {
		ErrorMessage( wDevNo );
	}
	ResultMessage( _T("TvO~܂B") );
	//----------------------------------------------------------
	// TvOXe[^X擾
	//----------------------------------------------------------
	if ( FALSE == Asb3000wGetStatus( wDevNo, &dwStatus, &dwNumData, &dwInitArg ) ) {
		ErrorMessage( wDevNo );
	}
	_stprintf_s( szbuf, 1024, _T( "TvOXe[^X擾\n" )
							  _T( "   TvOXe[^X : %xH\n" )
							  _T( "   ǃTvOf[^ : %d" ),
							  dwStatus, dwNumData );
	ResultMessage( szbuf );
	//----------------------------------------------------------
	// TvOf[^擾
	//----------------------------------------------------------
	if ( dwNumData > 0 ) {
		mpdwADData = (PDWORD)malloc( dwNumData * sizeof( DWORD ) );
		memset( mpdwADData, 0, dwNumData * sizeof( DWORD ) );
		if ( FALSE == Asb3000wGetData( wDevNo, dwNumData, mpdwADData ) ) {
			ErrorMessage( wDevNo );
		}
		_stprintf_s( szbuf, 1024, _T( "TvOf[^ %d 擾܂B\n\n" )
								  _T( "CH1 擾f[^\n" ),
								  dwNumData );
		for ( i = 4990; i <= 5009; i++ ) {	// 擾f[^\
			dwData = mpdwADData[ i ] & ADDATA_MASK_BIT;				// CH1
			//dwData = ( mpdwADData[ i ] >> 16 ) & ADDATA_MASK_BIT;	// CH2
			_stprintf_s( sztmp, 1024, _T( "   %d : %d ( %.2f )\n" ),
									  i + 1, dwData, VoltValue( dwData ) );
			_tcscat_s( szbuf, _countof(szbuf), sztmp );
		}
		_stprintf_s( sztmp, 1024, _T( "\n gKʒu : %d ̑O10\\n" )
								  _T( "    JbR͓dl(PmV)" ),
						 mSetMode.dwTrigPos );
		_tcscat_s( szbuf, _countof(szbuf), sztmp );
		ResultMessage( szbuf );
		free( mpdwADData );
	}
	//----------------------------------------------------------
	// I
	//----------------------------------------------------------
	SendMessage( mhWnd, WM_DESTROY, 0, 0 );
	return TRUE;
}
//-------------------------------------------------------------------
// dlZ
//-------------------------------------------------------------------
double VoltValue( DWORD addata )
{
	DWORD	dwvoltdiv;
	double	vfull;
	double	data;

	// Volt/div
	switch ( mSetMode.dwVoltDiv[ ASB3000_CH1 ] ) {
		case 0:
			dwvoltdiv = 10;
			break;
		case 1:
			dwvoltdiv = 20;
			break;
		case 2:
			dwvoltdiv = 50;
			break;
		case 3:
			dwvoltdiv = 100;
			break;
		case 4:
			dwvoltdiv = 200;
			break;
		case 5:
			dwvoltdiv = 500;
			break;
		case 6:
			dwvoltdiv = 1000;
			break;
		case 7:
			dwvoltdiv = 2000;
			break;
		default: dwvoltdiv = 0;
	}
	// tXP[d(v[u10:1)
	vfull = dwvoltdiv * ASB3000_PROBE_X10 * 10;
	// VoltPosftHgʒuɕύX
	data = addata + ( ( (double)ASB3000_RES_VERT / (double)ASB3000_RES_VOLTPOS )
					* ( ASB3000_DEF_VOLTPOS - mSetMode.dwVoltPos[ ASB3000_CH1 ] ) );
	// dlɊZ
	return ( ( vfull / ASB3000_RES_VERT * data ) - ( vfull / 2 ) );
}
//-------------------------------------------------------------------
// `f[^擾pXbhG[
//-------------------------------------------------------------------
BOOL DataThreadError( WPARAM wParam, LPARAM lParam )
{
	if ( ASB3000_DEVICE_ID == lParam ) {
		ErrorMessage( (WORD)wParam );
	}
	return TRUE;
}
//-------------------------------------------------------------------
// ʂ\
//-------------------------------------------------------------------
static void ResultMessage( LPCTSTR lpszbuf )
{
	MessageBox( NULL, lpszbuf, szTitle, MB_OK | MB_ICONINFORMATION );
}
//-------------------------------------------------------------------
// G[bZ[W\
//-------------------------------------------------------------------
static void ErrorMessage( WORD devno )
{
	DWORD dwRes = Asb3000wGetLastError( devno );
	TCHAR szbuf[ 50 ];

	if ( ASB3000_ERR_WRAPDLL == dwRes ) {
		_stprintf_s( szbuf, 50, _T( "DLL Load error" ) );
	}
	else {
		_stprintf_s( szbuf, 50, _T( "ASB-3000 error\n\n" )
								_T( "Error code : %.8xH" ),
								dwRes );
	}
	MessageBox( NULL, szbuf, szTitle, MB_OK | MB_ICONSTOP );
	exit(0);
}
