//===================================================================
// asb3000w.c
//-------------------------------------------------------------------
//
// ASB-3000 Wrapper
//
//===================================================================
#define STRICT

#include <windows.h>
#include <tchar.h>

#include "asb3000w.h"	// ASB-3000

static HMODULE hDLL = NULL;

// DLLGNX|[g֐AhX̊i[ϐ
static FARPROC pAsb3000GetVersion;
static FARPROC pAsb3000Create;
static FARPROC pAsb3000Close;
static FARPROC pAsb3000GetDevInfo;
static FARPROC pAsb3000GetMemSize;
static FARPROC pAsb3000SetMode;
static FARPROC pAsb3000StartSampling;
static FARPROC pAsb3000StopSampling;
static FARPROC pAsb3000GetStatus;
static FARPROC pAsb3000GetData;
static FARPROC pAsb3000GetLastError;

static BOOL Asb3000wGetDllExportFunc();

//-------------------------------------------------------------------
// 		DLL[h(񃍁[h̓G[Ƃ)
// 		
// ߂l	 True
//			s False
//-------------------------------------------------------------------
BOOL Asb3000wDllOpen( void )
{
	if ( hDLL != NULL )
		return FALSE;
	hDLL = (HMODULE)LoadLibrary( ASB3000_DLL_NAME );
	if ( hDLL == NULL )
		return FALSE;
	if ( Asb3000wGetDllExportFunc() == FALSE ) {
		Asb3000wDllClose();	// DLLGNX|[g֐擾ɎŝDLLN[Y
		return FALSE;
	}
	return TRUE;
}
//-------------------------------------------------------------------
// 		DLLA[h(A[h̓G[Ƃ)
// 		
// ߂l	 True
//			s False
//-------------------------------------------------------------------
BOOL Asb3000wDllClose( void )
{
	if ( hDLL == NULL )
		return TRUE;
	if ( FreeLibrary( hDLL ) == TRUE ) {
		hDLL = NULL;
		return TRUE;
	}
	return FALSE;
}
//-------------------------------------------------------------------
// 		o[W擾
// 		pdwDrvVer   hCõo[Wi[̈ւ̃|C^
//			pdwDllVer   DLL̃o[Wi[̈ւ̃|C^
// ߂l	 True
//			s False
//-------------------------------------------------------------------
BOOL Asb3000wGetVersion( PDWORD pdwDrvVer, PDWORD pdwDllVer )
{
	if ( hDLL == NULL )
		return FALSE;

	return (*pAsb3000GetVersion)( pdwDrvVer, pdwDllVer );
}
//-------------------------------------------------------------------
// 		foCX̌
// 		pwDeviceNo  gpfoCX̃foCXʔԍi[̈ւ
//						|C^
//			hWnd        DLLo郁bZ[W󂯎EBhẼnh
//	        uWMBase     o郁bZ[WID̃x[Xl
// ߂l	 True
//			s False
//-------------------------------------------------------------------
BOOL Asb3000wCreate( PWORD pwDeviceNo, HWND hWnd, UINT uWMBase )
{
	if ( hDLL == NULL )
		return FALSE;

	return (*pAsb3000Create)( pwDeviceNo, hWnd, uWMBase );
}
//-------------------------------------------------------------------
// 		foCX̊J
// 		wDeviceNo   ΏۃfoCX̃foCXʔԍ
// ߂l	 True
//			s False
//-------------------------------------------------------------------
BOOL Asb3000wClose( WORD wDeviceNo )
{
	if ( hDLL == NULL )
		return FALSE;

	return (*pAsb3000Close)( wDeviceNo );
}
//-------------------------------------------------------------------
// 		foCX̎擾
// 		wDeviceNo   ΏۃfoCX̃foCXʔԍ
//			pDevInfo    foCXi[\̂ւ̃|C^
// ߂l	 True
//			s False
//-------------------------------------------------------------------
BOOL Asb3000wGetDevInfo( WORD wDeviceNo, TDEVINFO *pDevInfo )
{
	if ( hDLL == NULL )
		return FALSE;

	return (*pAsb3000GetDevInfo)( wDeviceNo, pDevInfo );
}
//-------------------------------------------------------------------
// 		TCY̎擾
// 		wDeviceNo   ΏۃfoCX̃foCXʔԍ
//			pdwMemSize  Ă郁TCYi[̈ւ̃|C^
// ߂l	 True
//			s False
//-------------------------------------------------------------------
BOOL Asb3000wGetMemSize( WORD wDeviceNo, PDWORD pdwMemSize )
{
	if ( hDLL == NULL )
		return FALSE;

	return (*pAsb3000GetMemSize)( wDeviceNo, pdwMemSize );
}
//-------------------------------------------------------------------
// 		TvO샂[h̐ݒ
// 		wDeviceNo   ΏۃfoCX̃foCXʔԍ
//			pSetMode    ݒi[Ă\̂̃|C^
// ߂l	 True
//			s False
//-------------------------------------------------------------------
BOOL Asb3000wSetMode( WORD wDeviceNo, TSETMODE *pSetMode )
{
	if ( hDLL == NULL )
		return FALSE;

	return (*pAsb3000SetMode)( wDeviceNo, pSetMode );
}
//-------------------------------------------------------------------
// 		TvO̊Jn
// 		wDeviceNo   ΏۃfoCX̃foCXʔԍ
//			dwInitArg	\(0w)
// ߂l	 True
//			s False
//-------------------------------------------------------------------
BOOL Asb3000wStartSampling( WORD wDeviceNo, DWORD dwInitArg )
{
	if ( hDLL == NULL )
		return FALSE;

	return (*pAsb3000StartSampling)( wDeviceNo, dwInitArg );
}
//-------------------------------------------------------------------
// 		TvO̒~
// 		wDeviceNo   ΏۃfoCX̃foCXʔԍ
// ߂l	 True
//			s False
//-------------------------------------------------------------------
BOOL Asb3000wStopSampling( WORD wDeviceNo )
{
	if ( hDLL == NULL )
		return FALSE;

	return (*pAsb3000StopSampling)( wDeviceNo );
}
//-------------------------------------------------------------------
// 		TvOXe[^X擾
// 		wDeviceNo   ΏۃfoCX̃foCXʔԍ
//			pdwStatus   TvOXe[^X
//			pdwNumData  ǃTvOf[^
//			pvInitArg   \
// ߂l	 True
//			s False
//-------------------------------------------------------------------
BOOL Asb3000wGetStatus( WORD wDeviceNo, PDWORD pdwStatus, PDWORD pdwNumData, PVOID pvInitArg )
{
	if ( hDLL == NULL )
		return FALSE;

	return (*pAsb3000GetStatus)( wDeviceNo, pdwStatus, pdwNumData, pvInitArg );
}
//-------------------------------------------------------------------
// 		TvOf[^擾
// 		wDeviceNo   ΏۃfoCX̃foCXʔԍ
//			dwNumData   擾f[^
//			pdwData     擾f[^i[̈ւ̃|C^
// ߂l	 True
//			s False
//-------------------------------------------------------------------
BOOL Asb3000wGetData( WORD wDeviceNo, DWORD dwNumData, PDWORD pdwData )
{
	if ( hDLL == NULL )
		return FALSE;

	return (*pAsb3000GetData)( wDeviceNo, dwNumData, pdwData );
}
//-------------------------------------------------------------------
// 		G[R[h̎擾
// 		wDeviceNo   ΏۃfoCX̃foCXʔԍ
// ߂l	G[R[h
//-------------------------------------------------------------------
DWORD Asb3000wGetLastError( WORD wDeviceNo )
{
	if ( hDLL == NULL )
		return ASB3000_ERR_WRAPDLL;

	return (*pAsb3000GetLastError)( wDeviceNo );
}

//-------------------------------------------------------------------
// 		DLLGNX|[g֐AhX̎擾
// 		
// ߂l	 TRUE
//			s FALSE
//-------------------------------------------------------------------
static BOOL Asb3000wGetDllExportFunc()
{
	// Asb3000GetVersioñAhX擾
	pAsb3000GetVersion = GetProcAddress( hDLL, _T( "Asb3000GetVersion" ) );
	if ( pAsb3000GetVersion == NULL )
		return FALSE;
	// Asb3000CreatẽAhX擾
	pAsb3000Create = GetProcAddress( hDLL, _T( "Asb3000Create" ) );
	if ( pAsb3000Create == NULL )
		return FALSE;
	// Asb3000ClosẽAhX擾
	pAsb3000Close = GetProcAddress( hDLL, _T( "Asb3000Close" ) );
	if ( pAsb3000Close == NULL )
		return FALSE;
	// Asb3000GetDevInfõAhX擾
	pAsb3000GetDevInfo = GetProcAddress( hDLL, _T( "Asb3000GetDevInfo" ) );
	if ( pAsb3000GetDevInfo == NULL )
		return FALSE;
	// Asb3000GetMemSizẽAhX擾
	pAsb3000GetMemSize = GetProcAddress( hDLL, _T( "Asb3000GetMemSize" ) );
	if ( pAsb3000GetMemSize == NULL )
		return FALSE;
	// Asb3000SetModẽAhX擾
	pAsb3000SetMode = GetProcAddress( hDLL, _T( "Asb3000SetMode" ) );
	if ( pAsb3000SetMode == NULL )
		return FALSE;
	// Asb3000StartSampling̃AhX擾
	pAsb3000StartSampling = GetProcAddress( hDLL, _T( "Asb3000StartSampling" ) );
	if ( pAsb3000StartSampling == NULL )
		return FALSE;
	// Asb3000StopSampling̃AhX擾
	pAsb3000StopSampling = GetProcAddress( hDLL, _T( "Asb3000StopSampling" ) );
	if ( pAsb3000StopSampling == NULL )
		return FALSE;
	// Asb3000GetStatus̃AhX擾
	pAsb3000GetStatus = GetProcAddress( hDLL, _T( "Asb3000GetStatus" ) );
	if ( pAsb3000GetStatus == NULL )
		return FALSE;
	// Asb3000GetDatãAhX擾
	pAsb3000GetData = GetProcAddress( hDLL, _T( "Asb3000GetData" ) );
	if ( pAsb3000GetData == NULL )
		return FALSE;
	// Asb3000GetLastError̃AhX擾
	pAsb3000GetLastError = GetProcAddress( hDLL, _T( "Asb3000GetLastError" ) );
	if ( pAsb3000GetLastError == NULL )
		return FALSE;

	return TRUE;	// SĂDLLGNX|[g֐AhX擾
}
